/* replication of Gourinchas and Parker (Econometrica 2002) */
/* sam schulhofer-wohl, 9/13/2013 */
#include <math.h>
#include <omp.h>
#include <mkl.h>
#include <mkl_vsl.h>
#include <ipp.h>
#include <iostream>
#include "gp_ssw_intel.h"
using namespace std;

int main()
{

  time_t rawtime;
  time(&rawtime);
  cout << "log opened at " << ctime(&rawtime) << endl;
  
  /* 1. DECLARATIONS */

  /* start timing */
  double timer_start = omp_get_wtime();
  
  /* counters */
  long int i, j;
  
  /* basic parameters of the problem */
  struct problem myproblem __attribute__((aligned));
  struct problempart2 myproblempart2 __attribute__((aligned));
  myproblem.computeoptctrend=0;
  printf("number of threads = %d\n",GP_SSW_NUMTHREADS);
  printf("ruletol = %g\n",GP_SSW_RULETOL);
  printf("rulemaxiters = %d\n",GP_SSW_RULEMAXITERS);
  myproblem.gradtol_beta=GP_SSW_GRADTOL;
  printf("myproblem.gradtol_beta = %g\n",myproblem.gradtol_beta);
  myproblem.gradtol_rho=GP_SSW_GRADTOL;
  printf("myproblem.gradtol_rho = %g\n",myproblem.gradtol_rho);
  myproblem.gradtol_gamma0=GP_SSW_GRADTOL;
  printf("myproblem.gradtol_gamma0 = %g\n",myproblem.gradtol_gamma0);
  myproblem.gradtol_gamma1=GP_SSW_GRADTOL;
  printf("myproblem.gradtol_gamma1 = %g\n",myproblem.gradtol_gamma1);
  myproblem.gradtol_ytrend=GP_SSW_GRADTOL;
  printf("myproblem.gradtol_ytrend = %g\n",myproblem.gradtol_ytrend);
  myproblem.gradtol_ftrend=GP_SSW_GRADTOL;
  printf("myproblem.gradtol_ftrend = %g\n",myproblem.gradtol_ftrend);
  myproblem.gradtol_R=GP_SSW_GRADTOL;
  printf("myproblem.gradtol_R = %g\n",myproblem.gradtol_R);
  myproblem.gradtol_p=GP_SSW_GRADTOL;
  printf("myproblem.gradtol_p = %g\n",myproblem.gradtol_p);
  myproblem.gradtol_s_u=GP_SSW_GRADTOL;
  printf("myproblem.gradtol_s_u = %g\n",myproblem.gradtol_s_u);
  myproblem.gradtol_s_n=GP_SSW_GRADTOL;
  printf("myproblem.gradtol_s_n = %g\n",myproblem.gradtol_s_n);
  myproblem.useweights=1;
  myproblem.nparams1=4;
  myproblem.computeoptctrend=0;
  
  /* first-stage parameters */
  struct params1 params_stage1 __attribute__((aligned));
  params_stage1.R=1.0344;
  params_stage1.p=0.00302;
  double s2_u=0.0212;
  params_stage1.s_u=sqrt(s2_u);
  double s2_n=0.0440;
  params_stage1.s_n=sqrt(s2_n);
  params_stage1.wbar26=-2.7944810;
  params_stage1.s_w26=1.7838670;
  double * V1;
  V1=(double *)mkl_calloc(myproblem.nparams1*myproblem.nparams1,sizeof(double),64);
  V1[0]=pow(0.281/100,2); /* R */
  V1[1*myproblem.nparams1+1]=pow(.000764,2); /* p */
  double s2var[]={0.0000300973 /* transitory */, -0.0000132154, -0.0000132154, 0.0000086501 /* permanent */ };
  double svargrad[]={1/params_stage1.s_u,0,0,1/params_stage1.s_n};
  /* V1[2..3,2..3] = svargrad * s2var * svargrad */
  V1[2*myproblem.nparams1+2]=svargrad[0]*s2var[0]*svargrad[0];
  V1[2*myproblem.nparams1+3]=svargrad[0]*s2var[1]*svargrad[3];
  V1[3*myproblem.nparams1+2]=svargrad[3]*s2var[2]*svargrad[0];
  V1[3*myproblem.nparams1+3]=svargrad[3]*s2var[3]*svargrad[3];
  myproblem.V1=V1;
  
  /* optimization preferences */
  printf("minimizer relative tolerance: %g\n",GP_SSW_MINXTOL);
  
  /* 2. SIMULATE INCOME SHOCKS AND INITIAL WEALTH DISTRIBUTION */
  double *U, *N, *V, *w26;
  U=(double *)mkl_malloc(GP_SSW_T*GP_SSW_NSIMS*sizeof(double),64);
  N=(double *)mkl_malloc(GP_SSW_T*GP_SSW_NSIMS*sizeof(double),64);
  V=(double *)mkl_malloc(GP_SSW_T*GP_SSW_NSIMS*sizeof(double),64);
  w26=(double *)mkl_malloc(GP_SSW_NSIMS*sizeof(double),64);
  VSLStreamStatePtr randstream;
  vslNewStream(&randstream,VSL_BRNG_SFMT19937,7777777);
  vdRngGaussian(VSL_RNG_METHOD_GAUSSIAN_BOXMULLER2,randstream,GP_SSW_T*GP_SSW_NSIMS,U,0,1);
  vdRngGaussian(VSL_RNG_METHOD_GAUSSIAN_BOXMULLER2,randstream,GP_SSW_T*GP_SSW_NSIMS,N,0,1);
  vdRngGaussian(VSL_RNG_METHOD_GAUSSIAN_BOXMULLER2,randstream,GP_SSW_NSIMS,w26,0,1);
  vdRngUniform(VSL_RNG_METHOD_UNIFORM_STD,randstream,GP_SSW_T*GP_SSW_NSIMS,V,0,1);
  vslDeleteStream(&randstream);
  struct siminputs simshocks __attribute__((aligned));
  simshocks.U=U;
  simshocks.N=N;
  simshocks.V=V;
  simshocks.w26=w26;
  
  
  /* 3. MAKE THE CASH-ON-HAND GRID */
  double *xgrid;
  xgrid=(double *)mkl_malloc(GP_SSW_NGRID*sizeof(double),64);
  xgrid[0]=GP_SSW_XMIN;
  for(i=1;i<=GP_SSW_NGRIDLOW;i++) xgrid[i]=xgrid[i-1]+GP_SSW_DELTALOW;
  for(i=GP_SSW_NGRIDLOW+1;i<GP_SSW_NGRID;i++) xgrid[i]=xgrid[i-1]+GP_SSW_DELTAHIGH;
  myproblempart2.xgrid=xgrid;
  myproblempart2.xgridleftinv=1.0/xgrid[0];
  myproblempart2.xgridrightinv=1.0/xgrid[GP_SSW_NGRID-1];

  
  /* 4. MAKE THE GAUSS-HERMITE INTEGRATION GRID */
  double *intgrid, *intweights, *intweightscross;
  intgrid=(double *)mkl_malloc(GP_SSW_NINTEGRATE*sizeof(double),64);
  intweights=(double *)mkl_malloc(GP_SSW_NINTEGRATE*sizeof(double),64);
  intweightscross=(double *)mkl_malloc(GP_SSW_NINTEGRATE*GP_SSW_NINTEGRATE*sizeof(double),64);
  gp_ssw_gauher(GP_SSW_NINTEGRATE,intgrid,intweights);
  myproblem.intgrid=intgrid;
  myproblempart2.intweights=intweights;
  for(i=0;i<GP_SSW_NINTEGRATE;i++) for(j=0;j<GP_SSW_NINTEGRATE;j++) intweightscross[i*GP_SSW_NINTEGRATE+j]=intweights[i]*intweights[j];
  myproblempart2.intweightscross=intweightscross;

  
  /* 5. SET UP THE EXPECTED INCOME POLYNOMIAL, FAMILY SIZE POLYNOMIAL, AND SMOOTHED EXPECTED INCOME AND FAMILY SIZE */
  /* income polynomial (order 5 (1 to 5) and constant)    */
/*  double polY[]={.32643678179,-.0148947085,0.00036342384,-4.411685e-06,2.056916e-08,6.801368713}; */
  double *polY;
  polY=(double *)mkl_malloc(6*sizeof(double),64);
  polY[0]=.32643678179;
  polY[1]=-.0148947085;
  polY[2]=0.00036342384;
  polY[3]=-4.411685e-06;
  polY[4]=2.056916e-08;
  polY[5]=6.801368713;
  double * agep_work, * agep;
  agep=(double *)mkl_malloc((GP_SSW_T+1)*6*sizeof(double),64);
  agep_work=(double *)mkl_malloc((GP_SSW_T+1)*sizeof(double),64);
  ippsSet_64f(1.0,agep_work,GP_SSW_T+1);
  unsigned int ii, jj;
  for(ii=0;ii<GP_SSW_T+1;ii++) agep[ii*6+5]=1.0;
  for(jj=0;jj<5;jj++) {
    for(ii=0;ii<GP_SSW_T+1;ii++) {
      agep_work[ii]=agep_work[ii]*(26.0+(double)ii);
      agep[ii*6+jj]=agep_work[ii];
    }
  }
  double * y;
  y=(double *)mkl_calloc(GP_SSW_T+1,sizeof(double),64);
  cblas_dgemv(CblasRowMajor,CblasNoTrans,GP_SSW_T+1,6,1.0,agep,6,polY,1,0.0,y,1);
  mkl_free(polY);
  for(i=0;i<GP_SSW_T+1;i++) y[i]=exp(y[i]);
  params_stage1.y=y;
  double * g;
  g=(double *)mkl_malloc(GP_SSW_T*sizeof(double),64);
  for(i=0;i<GP_SSW_T;i++) g[i]=y[i+1]/y[i];
  params_stage1.g=g;
  /* family polynomial (order 5 (1 to 5), no constant */
  double polF[]={0.13964975,-0.0047742190,8.5155210e-5,-7.9110880e-7,2.9789550e-9};
  double * f0_work;
  f0_work=(double *)mkl_malloc((GP_SSW_T+1)*sizeof(double),64);
  /* skip the constant by ignoring the last column of agep (5 instead of 6) */
  cblas_dgemv(CblasRowMajor,CblasNoTrans,GP_SSW_T+1,5,1.0,agep,6,&polF[0],1,0.0,f0_work,1);
  double * f0;
  f0=(double *)mkl_malloc(GP_SSW_T*sizeof(double),64);
  for(ii=0;ii<GP_SSW_T;ii++) f0[ii]=exp(f0_work[ii+1]-f0_work[ii]);
  params_stage1.f0=f0;
  mkl_free(agep);
  mkl_free(agep_work);
  mkl_free(f0_work);
  

  /* 6. READ CONSUMPTION PROFILE AND ACTUAL CONSUMPTION DATA */
  
	/* profiles */
  int profiles_rows=GP_SSW_T;
  int profiles_cols=3;
  int fail;
  double * xx1_data = (double *) mkl_malloc(profiles_rows*profiles_cols*sizeof(double),64);
  fail=readfile("data/profiles.csv",xx1_data);
  if(fail) {
    printf("could not read data/profiles.csv\n");
    return EXIT_FAILURE;
  }
  double * lcprofile;
  lcprofile=(double *)mkl_malloc(GP_SSW_T*sizeof(double),64);
  for(i=0;i<GP_SSW_T;i++) lcprofile[i]=log(xx1_data[i*profiles_cols+1]);
  myproblem.lcprofile=lcprofile;
  
	/* actual consumption observations */
  int cc_rows=38691;
  int cc_cols=2;
  double * cc_data = (double *) mkl_malloc(cc_rows*cc_cols*sizeof(double),64);
  fail=readfile("data/colct1a.csv",cc_data);
  if(fail) {
    printf("could not read data/colct1a.csv\n");
    return EXIT_FAILURE;
  }
  
  
  /* 7. CONSTRUCT WEIGHTING MATRIX */
	/* calculates the weights and number of observations	*/
  unsigned long int nextrowtocheck=0, startrow;
  double it[GP_SSW_T] __attribute__((aligned));
  double * omega1;
  omega1=(double *)mkl_calloc(GP_SSW_T*GP_SSW_T,sizeof(double),64);
  int vsl_p=1, vsl_n, vsl_xstorage = VSL_SS_MATRIX_STORAGE_ROWS, vsl_covstorage=VSL_SS_MATRIX_STORAGE_FULL;
  VSLSSTaskPtr vsltask;
  double * lCt;
  double vslvar, vslm;
  for(j=1;j<=GP_SSW_T;j++) {
    startrow=nextrowtocheck;
    it[j-1]=0;
    while(cc_data[nextrowtocheck*cc_cols]==j+25) {
      nextrowtocheck++;
      it[j-1]+=1;
      if(nextrowtocheck>=cc_rows) break;
    }
    vsl_n=nextrowtocheck-startrow;
    lCt=(double *)mkl_malloc(vsl_n*sizeof(double),64);
    for(i=0;i<vsl_n;i++) lCt[i]=log(cc_data[(startrow+i)*cc_cols+1]);
    vsldSSNewTask(&vsltask,&vsl_p,&vsl_n,&vsl_xstorage,lCt,0,0);
    vsldSSEditTask(vsltask,VSL_SS_ED_COV,&vslvar);
    vsldSSEditCovCor(vsltask,&vslm,&vslvar,&vsl_covstorage,0,0);
    vsldSSCompute(vsltask,VSL_SS_COV,VSL_SS_METHOD_FAST);
    vslSSDeleteTask(&vsltask);
    vslvar=vslvar * (  ((double)(vsl_n-1)) / ((double)(vsl_n))   );
    omega1[(j-1)*GP_SSW_T+(j-1)]=vslvar;
    mkl_free(lCt);
  }
  vsl_n=GP_SSW_T;
  vsldSSNewTask(&vsltask,&vsl_p,&vsl_n,&vsl_xstorage,it,0,0);
  vsldSSEditTask(vsltask,VSL_SS_ED_MEAN,&myproblem.itm);
  vsldSSCompute(vsltask,VSL_SS_MEAN,VSL_SS_METHOD_FAST);
  vslSSDeleteTask(&vsltask);
  myproblem.taui=myproblem.itm/GP_SSW_NSIMS;
  double * W1;
  W1=(double *)mkl_malloc(GP_SSW_T*GP_SSW_T*sizeof(double),64);
  cblas_dcopy(GP_SSW_T*GP_SSW_T,omega1,1,W1,1);
  myproblem.omegamat=omega1;
  int lapack_i, lapack_j;
  LAPACKE_dpotrf(LAPACK_ROW_MAJOR,'U',GP_SSW_T,W1,GP_SSW_T);
  LAPACKE_dpotri(LAPACK_ROW_MAJOR,'U',GP_SSW_T,W1,GP_SSW_T); /* optimal weights, first stage, level        */
  for(lapack_i=1;lapack_i<GP_SSW_T;lapack_i++) for(lapack_j=0;lapack_j<lapack_i;lapack_j++) W1[lapack_i*GP_SSW_T+lapack_j]=W1[lapack_j*GP_SSW_T+lapack_i];
  myproblem.weightmat=W1;
  
  /* 8. REPORT RESULTS AT G-P POINT ESTIMATES */
  double test_timer_start, test_timer_end, z, oid2, oidall;
  
  /* robust estimates */
  struct params2 params_stage2_gp __attribute__((aligned));
  params_stage2_gp.beta=.9598;
  params_stage2_gp.rho=.5140;
  params_stage2_gp.gamma[0]=.0015;
  params_stage2_gp.gamma[1]=.0710;
  params_stage2_gp.ctrend=0;
  params_stage2_gp.ytrend=0;
  params_stage2_gp.ftrend=0;
  printf("myproblem.weightmat(0,0): %15.14g\n",myproblem.weightmat[0]);
  int ntestreps=1;
  z=gp_ssw_dist2(myproblem,myproblempart2,params_stage1,params_stage2_gp,simshocks,myproblem.weightmat);
  test_timer_start = omp_get_wtime();
  for(i=0;i<ntestreps;i++) z=gp_ssw_dist2(myproblem,myproblempart2,params_stage1,params_stage2_gp,simshocks,myproblem.weightmat);
  test_timer_end=omp_get_wtime();
  printf("results at Gourinchas and Parker (2002) robust point estimates:\n");
  printf("objective function: %15.14g\n",z);
  printf("time to calculate objective function: %g seconds\n\n",(test_timer_end-test_timer_start)/(double)ntestreps);
  double * grad, * var2, * varall, * efficientweights;
  grad=(double *)mkl_malloc(GP_SSW_T*4*sizeof(double),64);
  var2=(double *)mkl_malloc(4*4*sizeof(double),64);
  varall=(double *)mkl_malloc(4*4*sizeof(double),64);
  efficientweights=(double *)mkl_malloc(GP_SSW_T*GP_SSW_T*sizeof(double),64);
  test_timer_start = omp_get_wtime();
  gp_ssw_var2(myproblem,myproblempart2,params_stage1,params_stage2_gp,simshocks,0,0,0,0,grad,var2,varall,efficientweights,&oid2,&oidall); 
  test_timer_end=omp_get_wtime();
  printf("variance-covariance matrix (treating first stage as fixed):\n");
  for(i=0;i<4;i++) {
    for(j=0;j<4;j++) printf("%# 15.14g\t",var2[i*4+j]);
    printf("\n");
  }
  printf("chi-squared (treating first stage as fixed = %# 15.14g\n",oid2);
  printf("variance-covariance matrix (treating first stage as estimated):\n");
  for(i=0;i<4;i++) {
    for(j=0;j<4;j++) printf("%# 15.14g\t",varall[i*4+j]);
    printf("\n");
  }
  printf("chi-squared (treating first stage as estimated = %# 15.14g\n",oidall);
  printf("time to calculate variance-covariance matrix: %g seconds\n\n",test_timer_end-test_timer_start);

  /* efficient estimates -- no point reporting the chi-squared, but this is a useful starting point for estimation */
  struct params2 params_stage2_gp_efficient __attribute__((aligned));
  params_stage2_gp_efficient.beta=.9569;
  params_stage2_gp_efficient.rho=1.3969;
  params_stage2_gp_efficient.gamma[0]=5.6810e-6;
  params_stage2_gp_efficient.gamma[1]=.0613;
  params_stage2_gp_efficient.ctrend=0;
  params_stage2_gp_efficient.ytrend=0;
  params_stage2_gp_efficient.ftrend=0;
  printf("myproblem.weightmat(0,0): %15.14g\n",myproblem.weightmat[0]);
  test_timer_start = omp_get_wtime();
  z=gp_ssw_dist2(myproblem,myproblempart2,params_stage1,params_stage2_gp_efficient,simshocks,myproblem.weightmat);
  test_timer_end=omp_get_wtime();
  printf("results at Gourinchas and Parker (2002) efficient point estimates:\n");
  printf("objective function: %15.14g\n",z);
  printf("time to calculate objective function: %g seconds\n\n",test_timer_end-test_timer_start);
  
  /* 9. ESTIMATE */
  
#define estimatemsg(params2start,estctrend,estytrend,estftrend,opw) \
printf("estimating "); \
if(estctrend==1 || estytrend==1 || estftrend==1) { \
  printf("with trends in "); \
  if(estctrend==1) printf("c,"); \
  if(estytrend==1) printf("y,"); \
  if(estftrend==1) printf("f,"); \
  printf(" "); \
} \
else printf("baseline, ");\
if(opw==1) printf("efficient, "); else printf("robust, ");\
printf("starting from "#params2start ").\n"); \
time(&rawtime); \
cout << ctime(&rawtime) << endl << flush;

#define estimate(sfx,params2start,estctrend,estytrend,estftrend,opw) \
double * efficientweights_##sfx ; \
efficientweights_##sfx =(double *)mkl_malloc(GP_SSW_T*GP_SSW_T*sizeof(double),64); \
double fval_##sfx; \
printf("\n"); \
estimatemsg(params2start,estctrend,estytrend,estftrend,opw) \
const char * wmat_##sfx="weightmats/weightmat_" datestr "_" #sfx ;\
gp_ssw_outputweightmat(myproblem,wmat_##sfx);\
const char * prof_##sfx="profiles/prof_" datestr "_" #sfx ;\
struct params2 params_stage2_##sfx##_out __attribute__((aligned)); \
params_stage2_##sfx##_out = gp_ssw_optimize(myproblem,myproblempart2,params_stage1,params2start,simshocks,estctrend,estytrend,estftrend,efficientweights_##sfx,opw,&fval_##sfx,prof_##sfx);\
printf("done "); \
estimatemsg(params2start,estctrend,estytrend,estftrend,opw) \
printf(" --- \n\n\n");

#define estimate_nokeep(sfx,params2start,estctrend,estytrend,estftrend,opw) \
double * efficientweights_##sfx ; \
efficientweights_##sfx =(double *)mkl_malloc(GP_SSW_T*GP_SSW_T*sizeof(double),64); \
double fval_##sfx; \
printf("\n"); \
estimatemsg(params2start,estctrend,estytrend,estftrend,opw) \
const char * wmat_##sfx="weightmats/weightmat_" datestr "_" #sfx ;\
gp_ssw_outputweightmat(myproblem,wmat_##sfx);\
const char * prof_##sfx="profiles/prof_" datestr "_" #sfx ;\
gp_ssw_optimize(myproblem,myproblempart2,params_stage1,params2start,simshocks,estctrend,estytrend,estftrend,efficientweights_##sfx,opw,&fval_##sfx,prof_##sfx);\
printf("done "); \
estimatemsg(params2start,estctrend,estytrend,estftrend,opw) \
printf(" --- \n\n\n");
  
  printf("\n\n\n\n\n---- BASELINE MODEL, ROBUST ESTIMATES ----\n\n");
  myproblem.weightmat=W1;  
  estimate(robust,params_stage2_gp,0,0,0,0);

  printf("\n\n\n\n\n---- MODEL WITH C TREND, ROBUST ESTIMATES ----\n\n");
  estimate(ctrend_robust,params_stage2_robust_out,1,0,0,0);

  printf("\n\n\n\n\n---- MODEL WITH Y TREND, ROBUST ESTIMATES ----\n\n");
  estimate(ytrend_robust,params_stage2_robust_out,0,1,0,0);

  printf("\n\n\n\n\n---- MODEL WITH F TREND, ROBUST ESTIMATES ----\n\n");
  estimate(ftrend_robust,params_stage2_robust_out,0,0,1,0);

  printf("\n\n\n\n\n---- MODELS WITH C AND Y TRENDS, ROBUST ESTIMATES ----\n\n");
  estimate(cytrend_robust,params_stage2_ctrend_robust_out,1,1,0,0);
  estimate(yctrend_robust,params_stage2_ytrend_robust_out,1,1,0,0);

  printf("\n\n\n\n\n---- MODELS WITH C AND F TRENDS, ROBUST ESTIMATES ----\n\n");
  estimate(cftrend_robust,params_stage2_ctrend_robust_out,1,0,1,0);
  estimate(fctrend_robust,params_stage2_ftrend_robust_out,1,0,1,0);

  printf("\n\n\n\n\n---- MODELS WITH Y AND F TRENDS, ROBUST ESTIMATES ----\n\n");
  estimate(yftrend_robust,params_stage2_ytrend_robust_out,0,1,1,0);
  estimate(fytrend_robust,params_stage2_ftrend_robust_out,0,1,1,0);

  printf("\n\n\n\n\n---- MODELS WITH C, Y, AND F TRENDS, ROBUST ESTIMATES ----\n\n");
  estimate_nokeep(cyftrend_robust,params_stage2_cytrend_robust_out,1,1,1,0);
  estimate_nokeep(ycftrend_robust,params_stage2_yctrend_robust_out,1,1,1,0);
  estimate_nokeep(cfytrend_robust,params_stage2_cftrend_robust_out,1,1,1,0);
  estimate_nokeep(fcytrend_robust,params_stage2_fctrend_robust_out,1,1,1,0);
  estimate_nokeep(yfctrend_robust,params_stage2_yftrend_robust_out,1,1,1,0);
  estimate_nokeep(fyctrend_robust,params_stage2_fytrend_robust_out,1,1,1,0);

  /* 10. FINISH */
  
  /* free memory */
  mkl_free(U);
  mkl_free(N);
  mkl_free(V);
  mkl_free(w26);
  mkl_free(xgrid);
  mkl_free(intgrid);
  mkl_free(intweights);
  mkl_free(intweightscross);
  mkl_free(y);
  mkl_free(g);
  mkl_free(f0);
  mkl_free(lcprofile);
  mkl_free(grad);
  mkl_free(var2);
  mkl_free(varall);
  mkl_free(xx1_data);
  mkl_free(cc_data);
  mkl_free(omega1);
  mkl_free(W1);
  mkl_free(V1);
  
  /* done */
  printf("\n\n");
  printf("done in %g seconds.\n",omp_get_wtime() - timer_start);
  time(&rawtime);
  cout << "log closed at " << ctime(&rawtime) << endl;
  return 0;
  
}
