/* replication of Gourinchas and Parker (Econometrica 2002) */
/* sam schulhofer-wohl, 9/13/2013 */
/* calculates the distance between average consumption and average simulated profiles */
#include <mkl.h>
#include <ipp.h>
#include "gp_ssw_intel.h"
using namespace std;
#include <stdio.h>

/* weightmat: GP_SSW_T x GP_SSW_T */

double gp_ssw_dist2(const struct problem myproblem, const struct problempart2 myproblempart2, const struct params1 params_stage1, const struct params2 params_stage2, const struct siminputs simshocks, double * weightmat)
{
  double dd;
  double * d1;
  d1=(double *)mkl_malloc(GP_SSW_T*sizeof(double),64);
  
  /* compute moments */
  gp_ssw_momentvec(myproblem,myproblempart2,params_stage1,params_stage2,simshocks,d1);
  
  /* compute distance */
  if(myproblem.useweights==0) { /* no weights */
    ippsDotProd_64f(d1,d1,GP_SSW_T,&dd);
  }
  else {
    double work1[GP_SSW_T] __attribute__((aligned));
    cblas_dgemv(CblasRowMajor,CblasNoTrans,GP_SSW_T,GP_SSW_T,1.0,weightmat,GP_SSW_T,d1,1,0,work1,1);
    ippsDotProd_64f(d1,work1,GP_SSW_T,&dd);
  }
  
  mkl_free(d1);
  return dd;

}

