/* replication of Gourinchas and Parker (Econometrica 2002) */
/* sam schulhofer-wohl, 9/13/2013 */
/* wrapper for gp_ssw_dist2 for passing to an nlopt minimizer */
#include <iostream>
#include "gp_ssw_intel.h"
using namespace std;

double gp_ssw_dist2_wrapper(unsigned /* n */, const double *x, double *grad, void *params)
{
  /* retrieve parameters */
  struct dist2params *p = (struct dist2params *)params;
  struct params2 params_stage2  __attribute__((aligned));
  params_stage2.beta=x[0];
  params_stage2.rho=x[1];
  params_stage2.gamma[0]=x[2]*x[3]; /* helps impose gamma0<gamma1 */
  params_stage2.gamma[1]=x[3];
  int nextparam=4;
  if(p->optytrend==1) {
    params_stage2.ytrend=x[nextparam];
    nextparam++;
  }
  else params_stage2.ytrend=p->copyytrend;
  if(p->optftrend==1) {
    params_stage2.ftrend=x[nextparam];
    nextparam++;
  }
  else params_stage2.ftrend=p->copyftrend;
  if(p->optctrend==1) params_stage2.ctrend=0;
  else params_stage2.ctrend=p->copyctrend;

  /* compute gradient if desired */
#define dograd_gradwrapper(par,partarget,col) {\
my_params_stage2=params_stage2; \
hi=params_stage2.partarget+tols.par/2; \
my_params_stage2.partarget=hi; \
fhi=gp_ssw_dist2(p->myproblem,p->myproblempart2,p->params_stage1,my_params_stage2,p->simshocks,(p->myproblem).weightmat); \
lo=params_stage2.partarget-tols.par/2; \
my_params_stage2.partarget=lo; \
flo=gp_ssw_dist2(p->myproblem,p->myproblempart2,p->params_stage1,my_params_stage2,p->simshocks,(p->myproblem).weightmat); \
grad[col]=(fhi-flo)/(hi-lo); \
p->evalsincludinggrad+=2; \
}
  if(grad) {
    /* retrieve tolerances */
    struct gradtol tols;
    tols.beta=p->myproblem.gradtol_beta;
    tols.rho=p->myproblem.gradtol_rho;
    tols.gamma0=p->myproblem.gradtol_gamma0;
    tols.gamma1=p->myproblem.gradtol_gamma1;
    tols.ytrend=p->myproblem.gradtol_ytrend;
    tols.ftrend=p->myproblem.gradtol_ftrend;
    /* calculate gradient */
    double fhi, flo;
    volatile double hi, lo;
    struct params2 my_params_stage2  __attribute__((aligned));
    dograd_gradwrapper(beta,beta,0);
    dograd_gradwrapper(rho,rho,1);
    dograd_gradwrapper(gamma0,gamma[0],2);
    dograd_gradwrapper(gamma1,gamma[1],3);
    nextparam=4;
    if(p->optytrend==1) {
      dograd_gradwrapper(ytrend,ytrend,nextparam);
      nextparam++;
    }
    if(p->optftrend==1) {
      dograd_gradwrapper(ftrend,ftrend,nextparam);
      nextparam++;
    }
  }
  
  /* show status and return function value */
  int j;
  double f=gp_ssw_dist2(p->myproblem,p->myproblempart2,p->params_stage1,params_stage2,p->simshocks,(p->myproblem).weightmat);
  p->evalsincludinggrad++;
  if(f<=p->best) {
    p->best=f;
    for(j=0;j<nextparam;j++) printf("%# 6.5g ",x[j]);
    printf("f() = %# 15.14g evals incl. gradient = %d",f,p->evalsincludinggrad);
    cout << flush << endl;
    }
  return f;
}


