/* replication of Gourinchas and Parker (Econometrica 2002) */
/* sam schulhofer-wohl, 9/13/2013 */
#include <math.h>
#include <float.h>
#include <ipp.h>
#include "gp_ssw_intel.h"

/* marginal utility of the CRRA form. Uses a first order approximation if c<em
 */
double gp_ssw_du(const double c, const double negr0)
{
  /*
  const double em=1e-4, eminv=1e4, lnem=-9.210340371976;
  double cdiff=c-em;
  if(cdiff>=0) return pow(c,negr0);
  else return exp(negr0*lnem)*(1+negr0*cdiff*eminv);
  */
  return pow(c,negr0);
}


/* Euler equation residual. last working period.
 **	cs	: normalized current consumption
 **	x	: initial cash on hand
 ** g: growth in expected permanent income
 */
double __declspec(const) euler_last(const double cs, const double x, const double g, const struct params2 params_stage2, const struct eulerprecalc precalc)
{
  double c=cs*x;
	double z=gp_ssw_du(c,precalc.negrho)-precalc.Rbeta*gp_ssw_du((precalc.g1R*(x-c)+params_stage2.gamma[0])/g,precalc.negrho);
  if(isinf(z)) return DBL_MAX;
  else return z;
}

/* Euler equation residual.
 **	cs	: normalized current consumption
 **	x	: initial cash on hand
 ** g: growth in expected permanent income
 */
double __declspec(const) euler_1(const double cs, const double x, const struct problempart2 myproblempart2, const struct eulerprecalc precalc)
{
  int i, j, pos, posa;
  double I1,I2,posd;

  /* useful calculations */
  const double c=cs*x;
  const double const1=(x-c)*precalc.R_div_gf;
  double fdata[GP_SSW_NINTEGRATE] __attribute__((aligned(16)));
  ippsMulC_64f(precalc.invexpconst2Xgrid,const1,fdata,GP_SSW_NINTEGRATE);
  double workdata[GP_SSW_NINTEGRATE] __attribute__((aligned(16)));
  double fexpdata[GP_SSW_NINTEGRATE] __attribute__((aligned(16)));
  
  /* E[u'(c)|y'=0] */
  if(c==x) I1=DBL_MAX;
  else {
    #pragma ivdep
    for(j=0;j<GP_SSW_NINTEGRATE;j++) {
      /* interpolate consumption rule at these possible values and put into workdata */
      if(fdata[j]<GP_SSW_XMIN) workdata[j]=fdata[j]*precalc.ratioleft*precalc.gXexpconst2Xgrid[j];
      else if(fdata[j]>GP_SSW_XMAX) workdata[j]=fdata[j]*precalc.ratioright*precalc.gXexpconst2Xgrid[j];
      else if(fdata[j]<GP_SSW_XINT) {
        posd=(fdata[j]-GP_SSW_XMIN)*GP_SSW_INVDELTALOW;
        pos=(int)posd;
        workdata[j]=(precalc.rulerow[pos]+(posd-(double)pos)*(precalc.rulediffs[pos]))*precalc.gXexpconst2Xgrid[j];
      }
      else {
        posd=(fdata[j]-GP_SSW_XINT)*GP_SSW_INVDELTAHIGH;
        posa=(int)posd;
        pos=posa+GP_SSW_NGRIDLOW;
        workdata[j]=(precalc.rulerow[pos]+(posd-(double)posa)*(precalc.rulediffs[pos]))*precalc.gXexpconst2Xgrid[j];
      }
    }
    /* marginal utility */
    ippsPowx_64f_A53(workdata,precalc.negrho,fexpdata,GP_SSW_NINTEGRATE);
    ippsDotProd_64f(fexpdata,myproblempart2.intweights,GP_SSW_NINTEGRATE,&I1);
    I1=I1*precalc.I1mult;
  }
  
  /* E[u'(c)|y'>0] */
  double work3data[GP_SSW_NINTEGRATESQ] __attribute__((aligned(16)));
  double f2;
  /* at each possible value of cash on hand next period, interpolate consumption rule */
  int start=0;
  #pragma ivdep
  for(i=0;i<GP_SSW_NINTEGRATE;i++) {
    #pragma ivdep
    for(j=0;j<GP_SSW_NINTEGRATE;j++) {
      f2=fdata[i]+precalc.expconst3Xgrid[j];
      if(f2<GP_SSW_XMIN) work3data[start+j]=f2*precalc.ratioleft*precalc.gXexpconst2Xgrid[i];
      else if(f2>GP_SSW_XMAX) work3data[start+j]=f2*precalc.ratioright*precalc.gXexpconst2Xgrid[i];
      else if(f2<GP_SSW_XINT) {
          posd=(f2-GP_SSW_XMIN)*GP_SSW_INVDELTALOW;
          pos=(int)posd;
          work3data[start+j]=(precalc.rulerow[pos]+(posd-pos)*(precalc.rulediffs[pos]))*precalc.gXexpconst2Xgrid[i];
        }
      else {
        posd=(f2-GP_SSW_XINT)*GP_SSW_INVDELTAHIGH;
        posa=(int)posd;
        pos=posa+GP_SSW_NGRIDLOW;
        work3data[start+j]=(precalc.rulerow[pos]+(posd-posa)*(precalc.rulediffs[pos]))*precalc.gXexpconst2Xgrid[i];
      } 
    }
    start=start+GP_SSW_NINTEGRATE;
  }
  /* marginal utility and integration (written so it will vectorize) */
  double f2expdata[GP_SSW_NINTEGRATESQ] __attribute__((aligned(16)));
  ippsPowx_64f_A53(work3data,precalc.negrho,f2expdata,GP_SSW_NINTEGRATESQ);
  ippsDotProd_64f(f2expdata,myproblempart2.intweightscross,GP_SSW_NINTEGRATESQ,&I2);
  I2=I2*precalc.I2mult;

  /* finished */
  double z=gp_ssw_du(c,precalc.negrho)-(I2+I1);
  if(isinf(z)) return DBL_MAX;
  else return z;
}



