/* replication of Gourinchas and Parker (Econometrica 2002) */
/* sam schulhofer-wohl, 9/13/2013 */
/* returns the abscissas and weights of the Gauss-Hermite quadrature. From Numerical Recipes in C
** INPUT:
**	n: order of the quadrature
** OUTPUT:
**	x: nodes of quadrature
**	w: weights
*/

#include <math.h>
#include <float.h>
#include <stdio.h>
#include "gp_ssw_intel.h"

void gp_ssw_gauher(int n, double *x, double *w)
{
  const double eps=3.0e-14, pima=pow(M_PI,-0.25), m=(n+1)/2;
  const int maxit=10;
  int i, its, j;
  double z, p1, p2, p3, pp, z1;
  for(i=0;i<n;i++) {
    x[i]=0;
    w[i]=0;
  }
  for(i=1;i<=m;i++) {
    if(i==1) z=sqrt(2*n+1)-1.85575*exp(-0.16667*log(2*n+1));
    else if(i==2) z= z-1.14*(exp(0.426*log(n)))/z;
    else if(i==3) z= 1.86*z-0.86*x[0];
    else if(i==4) z= 1.91*z-0.91*x[1];
    else z=2.0*z-x[i-3];
    its=1;
    while(its<=maxit) {
      p1=pima;
      p2=0.0;
      for(j=1;j<=n;j++) {
        p3=p2;
        p2=p1;
        p1=z*sqrt(2.0/j)*p2-sqrt((j-1.0)/j)*p3;
      }
      pp=sqrt(2*n)*p2;
      z1=z;
      z=z1-p1/pp;
      if(fabs(z-z1)<=eps) break;
      its++;
    }
    if(its>maxit) printf("too many iterations in gauher\n");
    x[i-1]=z;
    x[n-i]=-z;
    w[i-1]=2.0/(pp*pp);
    w[n-i]=w[i-1];
  }
}

