/* replication of Gourinchas and Parker (Econometrica 2002) */
/* sam schulhofer-wohl, 9/13/2013 */
/* produce a vector of the difference between average consumption and average simulated profiles at each age*/
#include <math.h>
#include <mkl.h>
#include <ipp.h>
#include <stdlib.h>
#include "gp_ssw_intel.h"

/* moments: Tx1 vector. must exist on input. on output, has the moment differences. */
void gp_ssw_momentvec(const struct problem myproblem, const struct problempart2 myproblempart2, const struct params1 params_stage1, const struct params2 params_stage2, const struct siminputs simshocks, double *moments)
{
  
  int i;
  
  /* compute the consumption rule */
  double * rule;
  rule=(double *)mkl_malloc((GP_SSW_T+1)*GP_SSW_NGRID*sizeof(double),64);
  gp_ssw_rule(myproblem,myproblempart2,params_stage1,params_stage2,rule);

  /* simulate profiles using the rule */
  double * csim;
  csim=(double *)mkl_malloc(GP_SSW_T*sizeof(double),64);
  gp_ssw_sim(myproblempart2,simshocks,params_stage1,params_stage2,rule,csim,0,0);

  double ctrend;
  if(myproblem.computeoptctrend==1) {
    /* compute optimal ctrend */
#pragma ivdep
    for(i=0;i<GP_SSW_T;i++) moments[i]=log(csim[i])-myproblem.lcprofile[i];
    double z;
    ippsDotProd_64f(moments,myproblem.optcnum,GP_SSW_T,&z);
    ctrend=z/myproblem.optcden;
  }
  else {
    /* use whatever ctrend is in the parameter vector */
    ctrend=params_stage2.ctrend;
  }

  /* compute intercept so that adding a slope ctrend doesn't change mean of consumption profile */
  /* we add ctrend*i to element i, for i=0 to T-1 */
  /* so in total, we add ctrend*(T-1)*T/2 */
  /* that raises the mean by ctrend*(T-1)/2 */
  double offset=0.5*ctrend*((double)(GP_SSW_T-1));
#pragma ivdep
  for(i=0;i<GP_SSW_T;i++) moments[i]=log(csim[i])-(myproblem.lcprofile[i]+ctrend*i-offset);
  
  mkl_free(rule);
  mkl_free(csim);

}

