/* replication of Gourinchas and Parker (Econometrica 2002) */
/* sam schulhofer-wohl, 9/13/2013 */
/* output profiles: simulated consumption, age profile in the data tilted by the trend, income profile in the data tilted by the trend */
#include <math.h>
#include <mkl.h>
#include <stdio.h>
#include "gp_ssw_intel.h"

void gp_ssw_outputprofiles(const struct problem myproblem, const struct problempart2 myproblempart2, const struct params1 params_stage1, const struct params2 params_stage2, const struct siminputs simshocks, const char *filename)
{
  
  int i;
  
  /* compute the consumption rule */
  double * rule;
  rule=(double *)mkl_malloc((GP_SSW_T+1)*GP_SSW_NGRID*sizeof(double),64);
  gp_ssw_rule(myproblem,myproblempart2,params_stage1,params_stage2,rule);

  /* simulate profiles using the rule */
  double * csim, * ysim;
  csim=(double *)mkl_malloc(GP_SSW_T*sizeof(double),64);
  ysim=(double *)mkl_malloc(GP_SSW_T*sizeof(double),64);
  gp_ssw_sim(myproblempart2,simshocks,params_stage1,params_stage2,rule,csim,ysim,1);
  
  double ctrend;
  if(myproblem.computeoptctrend==1) {
    /* compute optimal ctrend */
    double moments[GP_SSW_T] __attribute__((aligned));
    #pragma ivdep
    for(i=0;i<GP_SSW_T;i++) moments[i]=log(csim[i])-myproblem.lcprofile[i];
    ctrend=cblas_ddot(GP_SSW_T,moments,1,myproblem.optcnum,1)/myproblem.optcden;
  }
  else {
    /* use whatever ctrend is in the parameter vector */
    ctrend=params_stage2.ctrend;
  }
  
  /* compute intercept so that adding a slope ctrend doesn't change mean of consumption profile */
  /* we add ctrend*i to element i, for i=0 to T-1 */
  /* so in total, we add ctrend*(T-1)*T/2 */
  /* that raises the mean by ctrend*(T-1)/2 */
  double offset=0.5*ctrend*((double)(GP_SSW_T-1));
  
  /* output */
  FILE * myfile=fopen(filename,"w");
  fprintf(myfile,"age,csim,cprofile,yprofile\n");
  for(i=0;i<GP_SSW_T;i++)
    fprintf(myfile,"%d,%15.14g,%15.14g,%15.14g\n",
            26+i,
            csim[i],
            exp(myproblem.lcprofile[i]+ctrend*i-offset),
            ysim[i]);
  fclose(myfile);
  
  mkl_free(rule);
  mkl_free(csim);
  mkl_free(ysim);

}

