log using graph.log, replace
version 13.0

*make graphs for "The Age-Time-Cohort Problem and the Identification of Structural Parameters in Life-Cycle Models" by Sam Schulhofer-Wohl

local ests "20130918d"

insheet using "prof_`ests'_robust.", names comma clear
rename (yprofile c*) base_=
save prof_`ests'.dta, replace

foreach p in fcy {
  insheet using "prof_`ests'_`p'trend_robust.", names comma clear
  rename (yprofile c*) `p'trend_=
  merge 1:1 age using prof_`ests'.dta, nogen assert(match)
  save prof_`ests'.dta, replace
  }

list *_cprofile, noobs clean
sum *_cprofile
list *_csim, noobs clean
sum *_csim
list *_yprofile, noobs clean
sum *_yprofile

lab var base_cprofile "data (as normalized by Gourinchas and Parker, 2002)"
lab var base_csim "estimated model (standard method)"
lab var fcytrend_cprofile "data (rotated according to new method)"
lab var fcytrend_csim "estimated model (new method)"
twoway connected base_cprofile base_csim fcytrend_cprofile fcytrend_csim age, ///
  scheme(my2up) lcolor(midblue midblue dkorange dkorange) lpattern(solid longdash solid shortdash) msymbol(O none D none) msize(medium medium medium medium) mcolor(midblue midblue dkorange dkorange) ///
  ylabel(15000 "$15,000" 18000 "$18,000" 21000 "$21,000" 24000 "$24,000",grid gmin gmax labsize(small)) ///
  xlabel(25(5)65,grid gmin gmax labsize(small)) xtitle("age",size(small)) legend(size(small) rows(4))
graph export fig2.eps, logo(off) replace
!epstopdf fig2.eps
twoway connected base_cprofile base_csim fcytrend_cprofile fcytrend_csim age, ///
  scheme(myslides) lcolor(midblue midblue dkorange dkorange) lpattern(solid longdash solid shortdash) msymbol(O none D none) msize(medium medium medium medium) mcolor(midblue midblue dkorange dkorange) ///
  ylabel(15000 "$15,000" 18000 "$18,000" 21000 "$21,000" 24000 "$24,000",grid gmin gmax labsize(medium)) ///
  xlabel(25(5)65,grid gmin gmax labsize(medium)) xtitle("age",size(medium)) legend(size(medium) rows(4)) ytitle("consumption",size(medium))
graph export fig2slide.eps, logo(off) replace
!epstopdf fig2slide.eps

foreach v in base_cprofile base_csim fcytrend_csim {
  gen l`v'=log(`v')
  qui reg l`v' age
  predict rl`v', resid
  }
lab var rlbase_cprofile "data"
lab var rlbase_csim "estimated model (standard method)"
lab var rlfcytrend_csim "estimated model (new method)"
twoway line rlbase_cprofile rlbase_csim rlfcytrend_csim age, ///
  scheme(my2up) lcolor(black midblue dkorange) lpattern(solid longdash shortdash) ///
  ylabel(,grid gmin gmax labsize(small)) xlabel(25(5)65,grid gmin gmax labsize(small)) xtitle("age",size(small)) ///
  legend(size(small) rows(3))
graph export fig1.eps, logo(off) replace
!epstopdf fig1.eps
twoway line rlbase_cprofile rlbase_csim rlfcytrend_csim age, ///
  scheme(myslides) lcolor(black midblue dkorange) lpattern(solid longdash shortdash) ///
  ylabel(,grid gmin gmax labsize(medium)) xlabel(25(5)65,grid gmin gmax labsize(medium)) xtitle("age",size(medium)) ///
  ytitle("detrended ln(consumption)", size(medium)) legend(size(medium) rows(3))
graph export fig1slide.eps, logo(off) replace
!epstopdf fig1slide.eps

log close
exit
